package com.pancik.libgdx.zumpatalentov.engine;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input;
import com.badlogic.gdx.graphics.OrthographicCamera;
import com.badlogic.gdx.math.Vector2;
import com.pancik.libgdx.zumpatalentov.MainApplicationListener;
import com.pancik.libgdx.zumpatalentov.SoundData;
import com.pancik.libgdx.zumpatalentov.engine.component.EntityManager;
import com.pancik.libgdx.zumpatalentov.engine.component.LevelTileManager;
import com.pancik.libgdx.zumpatalentov.engine.component.ParticleManager;
import com.pancik.libgdx.zumpatalentov.engine.component.PlayerManager;
import com.pancik.libgdx.zumpatalentov.engine.component.TileManager;
import com.pancik.libgdx.zumpatalentov.engine.particle.Particle;
import com.pancik.libgdx.zumpatalentov.engine.player.AIPlayer;
import com.pancik.libgdx.zumpatalentov.engine.player.LocalPlayer;
import com.pancik.libgdx.zumpatalentov.engine.player.Player;
import com.pancik.libgdx.zumpatalentov.engine.player.tutorial.TextTutorial;
import com.pancik.libgdx.zumpatalentov.engine.player.tutorial.Tutorial;
import com.pancik.libgdx.zumpatalentov.entity.Entity;
import com.pancik.libgdx.zumpatalentov.util.KeyChangedEvent;
import com.pancik.libgdx.zumpatalentov.util.MouseEvent;
import com.pancik.libgdx.zumpatalentov.util.MouseScrolledEvent;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.logging.Logger;

public class Engine {

    public static interface EngineCallback {
        public void hasWon();
        public void hasLost();
        public void exited();
    }

    public class EngineControls {

        public EntityManager getEntityManager() {
            return entityManager;
        }

        public TileManager getTileManager() {
            return tileManager;
        }

        public Level.PlayerDescription getPlayerDescription(int playerNumber) {
            return level.getPlayerDescription(playerNumber);
        }

        public void addEntity(Entity e) {
            getEntityManager().addEntity(e);
        }

        public void addParticle(Particle p) {
            particleManager.addParticle(p);
        }

        public void addDelayedAction(DelayedAction action) {
            delayedActions.add(action);
        }

        public OrthographicCamera getUICamera() {
            return uiCamera;
        }

        public OrthographicCamera getGameCamera() {
            return gameCamera;
        }
    }
    protected OrthographicCamera gameCamera;
    protected OrthographicCamera uiCamera;
    protected EngineCallback callback;
    protected TileManager tileManager;
    protected EntityManager entityManager;
    protected PlayerManager playerManager;
    protected ParticleManager particleManager;
    protected Level level;
    protected Player localPlayer = null;
    protected boolean gameIsFinished = false;
    protected List<DelayedAction> delayedActions = new ArrayList<DelayedAction>();
    protected LinkedList<Tutorial> tutorials = new LinkedList<Tutorial>();
    protected int endCountdown = 300;
    protected boolean playerWon;

    public Engine(MainApplicationListener main, String levelFile, EngineCallback callback) {
        this.callback = callback;

        this.gameCamera = main.getGameCamera();
        this.gameCamera.zoom = 1.0f;

        this.uiCamera = main.getUICamera();

        try {
            this.level = new Level(levelFile);
        } catch (Exception ex) {
            Logger.getLogger(Engine.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        }

        try {
            this.tileManager = new LevelTileManager(level);
        } catch (Exception ex) {
            Logger.getLogger(Engine.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        }

        this.particleManager = new ParticleManager();
        this.entityManager = new EntityManager(this.tileManager);

        this.localPlayer = new LocalPlayer(0, 0, new EngineControls());
        this.localPlayer.loadFromDescription(level.getPlayerDescription(0));

        this.playerManager = new PlayerManager();
        this.playerManager.addPlayer(localPlayer);

        int playerCount = level.getPlayersCount();
        if (playerCount < 1) {
            System.out.println("This is some serious shit bro'.. You have less than one player on this map?");
        }
        for (int i = 1; i < playerCount; i++) {
            Player p = new AIPlayer(i, i, new EngineControls());
            p.loadFromDescription(level.getPlayerDescription(i));
            this.playerManager.addPlayer(p);
        }

        for (Level.PlayerDescription.TutorialDescription tutDesc : level.getPlayerDescription(0).tutorials) {
            this.tutorials.add(new TextTutorial(new EngineControls(), tutDesc.target.cpy(), tutDesc.text));
        }

        gameCamera.position.set(level.getPlayerDescription(0).getBaseSpawnPoint().x, level.getPlayerDescription(0).getBaseSpawnPoint().y, 0);
        SoundData.playEngineMusic();
    }

    public void tick() {
        if (tutorials.isEmpty()) {
            for (DelayedAction action : delayedActions) {
                if (!action.remove()) {
                    action.tick();
                }
            }

            for (DelayedAction action : delayedActions) {
                if (action.remove()) {
                    delayedActions.remove(action);
                    break;
                }
            }

            tileManager.tick();

            entityManager.tick();

            playerManager.tick();

            particleManager.tick();

            if (localPlayer != null) {
                Vector2 cameraPos = localPlayer.getCameraPosition();
                gameCamera.position.set(cameraPos.x, cameraPos.y, 0.0f);
            }
        } else {
            if (!tutorials.getFirst().isFinished()) {
                tutorials.getFirst().tick();
            }

            if (tutorials.getFirst().isFinished()) {
                tutorials.removeFirst();
            }
        }

        if (playerManager.isWinner(0)) {
            gameIsFinished = true;
            playerWon = true;
        }
        if (localPlayer.isDefeated()) {
            gameIsFinished = true;
            playerWon = false;
        }

        if (gameIsFinished) {
            endCountdown--;
            if (endCountdown <= 0) {
                if (playerWon) {
                    callback.hasWon();
                } else {
                    callback.hasLost();
                }
            }
        }
    }

    public void render() {
        tileManager.render();

        particleManager.earlyRender();

        entityManager.render();

        particleManager.lateRender();
    }

    public void renderUI() {
        localPlayer.renderUI();

        if (!tutorials.isEmpty()) {
            if (!tutorials.getFirst().isFinished()) {
                tutorials.getFirst().renderUI();
            }
        }
    }

    public void handle(MouseEvent me) {
        if (tutorials.isEmpty()) {
            if (localPlayer != null) {
                localPlayer.handle(me);
            }
        } else {
            tutorials.getFirst().handle(me);
        }
    }

    public void handle(MouseScrolledEvent mse) {
        if (tutorials.isEmpty()) {
            gameCamera.zoom += (float) (mse.getAmount()) / 12.0f;

            if (gameCamera.zoom < 0.25f) {
                gameCamera.zoom = 0.25f;
            }
            if (gameCamera.zoom > 2.0f) {
                gameCamera.zoom = 2.0f;
            }
        }
    }

    public void handle(KeyChangedEvent kce) {
        if (kce.getKeyPressed() == Input.Keys.ESCAPE) {
            callback.exited();
        }
    }
}
