package com.pancik.libgdx.zumpatalentov.engine;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.files.FileHandle;
import com.badlogic.gdx.math.Vector2;
import com.pancik.libgdx.zumpatalentov.engine.tile.Tile;
import java.io.BufferedReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.logging.Logger;
import org.lwjgl.util.Point;

public class Level {
    public static class WaypointPath {
        private List<Vector2> path = new ArrayList<Vector2>();
        
        public void addWaypoint(Vector2 waypoint) {
            path.add(waypoint);
        }
        
        public List<Vector2> getWaypoints() {
            return path;
        }
    }
    
    public static class PlayerDescription {
        public static class BuildingSpotInfo {
            public Vector2 pos;
            public int buildingsBuildable;

            public BuildingSpotInfo(Vector2 pos, int buildingsBuildable) {
                this.pos = pos;
                this.buildingsBuildable = buildingsBuildable;
            }
        }
        
        public static class TutorialDescription {
            public Vector2 target;
            public String[] text;

            public TutorialDescription(Vector2 target, String[] text) {
                this.target = target;
                this.text = text;
            }
        }
        
        private Vector2 startingPoint = null;
        private List<WaypointPath> waypointPaths = new ArrayList<WaypointPath>();
        private Vector2 baseSpawnPoint = null;
        private boolean baseSpawnPointFlipped = false;
        
        private List<BuildingSpotInfo> towerBuildingSpots = new ArrayList<BuildingSpotInfo>();
        private List<BuildingSpotInfo> baseBuildingSpots = new ArrayList<BuildingSpotInfo>();
        
        public List<TutorialDescription> tutorials = new LinkedList<TutorialDescription>();
        
        public Vector2 getStartingPoint() {
            return startingPoint.cpy();
        }

        public List<WaypointPath> getWaypointPaths() {
            return waypointPaths;
        }

        public Vector2 getBaseSpawnPoint() {
            return baseSpawnPoint.cpy();
        }

        public boolean isBaseSpawnPointFlipped() {
            return baseSpawnPointFlipped;
        }

        public List<BuildingSpotInfo> getTowerBuildingSpots() {
            return towerBuildingSpots;
        }

        public List<BuildingSpotInfo> getBaseBuildingSpots() {
            return baseBuildingSpots;
        }
        
        public void setStartingPoint(Vector2 startingPoint) {
            this.startingPoint = startingPoint;
        }

        public void setBaseSpawnPoint(Vector2 baseSpawnPoint, boolean flipped) {
            this.baseSpawnPoint = baseSpawnPoint;
            this.baseSpawnPointFlipped = flipped;
        }
        
        public void addWaypointPaths(List<WaypointPath> paths) {
            this.waypointPaths.addAll(paths);
        }
        
        public void addTowerBuildingSpot(Vector2 pos, int towersBuildable) {
            this.towerBuildingSpots.add(new BuildingSpotInfo(pos, towersBuildable));
        }
        
        public void addBaseBuildingSpot(Vector2 pos, int buildingsBuildable) {
            this.baseBuildingSpots.add(new BuildingSpotInfo(pos, buildingsBuildable));
        }
    }
    
    private int width = 0;
    private int height = 0;
    private Tile.Type[][] tileLayout;
    private Character[][] scriptLayout = null;
    private int playersCount = 0;
    private PlayerDescription[] playerDescriptions;
    
    public Level(String fileName) throws Exception {
        FileHandle fileInput = Gdx.files.local(fileName);
        
        if(fileInput.exists()) {
            BufferedReader input = new BufferedReader(fileInput.reader());
            String rawInput;
            int line = 0;
            try {
                while((rawInput = input.readLine()) != null) {
                    if(line == 0) {
                        //System.out.println("1 " + rawInput);
                        String[] parsed = rawInput.split(" ");
                        width = Integer.parseInt(parsed[0]);
                        height = Integer.parseInt(parsed[1]);
                        playersCount = Integer.parseInt(parsed[2]);
                        
                        tileLayout = new Tile.Type[height][width];
                        scriptLayout = new Character[height][width];
                        playerDescriptions = new PlayerDescription[playersCount];
                        for(int i = 0; i< playersCount; i++) {
                            playerDescriptions[i] = new PlayerDescription();
                        }
                    } else if(line > 0 && line <= height) {
                        //System.out.println("2 " + rawInput);
                        if(rawInput.length() == width) {
                            for(int i = 0; i < width; i++) {
                                Tile.Type type = null;

                                switch(rawInput.charAt(i)) {
                                    case '.':
                                        type = Tile.Type.GRASS;
                                        break;
                                    case ',':
                                        type = Tile.Type.FLOOR;
                                        break;
                                    case 'R':
                                        type = Tile.Type.ROAD;
                                        break;
                                    case 'W':
                                        type = Tile.Type.WALL;
                                        break;
                                    case '#':
                                        break;
                                    default:
                                        throw new Exception("Missing character recognition for: " + rawInput.charAt(i));
                                }
                                tileLayout[line - 1][i] = type;
                            }
                        }
                    } else if(line > height + 1 && line <= (height) * 2 + 1) {
                        //System.out.println("3 " + rawInput);
                        if(rawInput.length() == width) {
                            for(int i = 0; i < width; i++) {
                                scriptLayout[line - (height + 2)][i] = rawInput.charAt(i);
                            }
                        }
                    } else if(line > (height) * 2 + 1){
                        //System.out.println("4 " + rawInput);
                        if(rawInput.length() > 2) {
                            String[] parsed = rawInput.split(" ");
                            Point[] scriptPositions = getCharacterPosInScriptLayout(parsed[0].charAt(0));

                            for(Point scriptPos : scriptPositions) {
                                if(parsed[1].equals("spawn")) {
                                    int playerNumber = Integer.parseInt(parsed[2]);
                                    List<WaypointPath> paths = new ArrayList<WaypointPath>();

                                    String[] subparsed = parsed[3].split(";");
                                    for(String waypointList : subparsed) {
                                        String[] waypoints = waypointList.split(",");

                                        WaypointPath tempPath = new WaypointPath();
                                        for(String waypoint : waypoints) {
                                            Point waypointPos = getCharacterPosInScriptLayout(waypoint.charAt(0))[0];
                                            tempPath.path.add(new Vector2(waypointPos.getX() * Tile.TILE_SIZE, waypointPos.getY() * Tile.TILE_SIZE));
                                        }
                                        paths.add(tempPath);
                                    }

                                    playerDescriptions[playerNumber].setStartingPoint(new Vector2(scriptPos.getX() * Tile.TILE_SIZE, scriptPos.getY() * Tile.TILE_SIZE));
                                    playerDescriptions[playerNumber].addWaypointPaths(paths);
                                } else if(parsed[1].equals("base")) {
                                    int playerNumber = Integer.parseInt(parsed[2]);
                                    boolean flipped = Boolean.parseBoolean(parsed[3]);

                                    playerDescriptions[playerNumber].setBaseSpawnPoint(new Vector2((scriptPos.getX() * Tile.TILE_SIZE),
                                            (scriptPos.getY() * Tile.TILE_SIZE)), flipped);
                                } else if(parsed[1].equals("tower")) {
                                    int playerNumber = Integer.parseInt(parsed[2]);
                                    int towersBuildable = Integer.parseInt(parsed[3]);

                                    playerDescriptions[playerNumber].addTowerBuildingSpot(new Vector2((scriptPos.getX() * Tile.TILE_SIZE),
                                            (scriptPos.getY() * Tile.TILE_SIZE)), towersBuildable);
                                } else if(parsed[1].equals("building")) {
                                    int playerNumber = Integer.parseInt(parsed[2]);
                                    int buildingsBuildable = Integer.parseInt(parsed[3]);

                                    playerDescriptions[playerNumber].addBaseBuildingSpot(new Vector2((scriptPos.getX() * Tile.TILE_SIZE),
                                            (scriptPos.getY() * Tile.TILE_SIZE)), buildingsBuildable);
                                } else if(parsed[1].equals("textTutorial")) {
                                    int playerNumber = Integer.parseInt(parsed[2]);
                                    String temp = parsed[3];
                                    String[] texts = temp.split(";");
                                    String[] goodTexts = new String[texts.length];
                                    for(int i = 0; i < texts.length; i++) {
                                        goodTexts[i] = texts[i].replace('_', ' ');
                                    }

                                    playerDescriptions[playerNumber].tutorials.add(new PlayerDescription.TutorialDescription(new Vector2((scriptPos.getX() * Tile.TILE_SIZE),
                                            (scriptPos.getY() * Tile.TILE_SIZE)), goodTexts));
                                }
                            }
                        }
                    }
                    line++;
                }
            } catch (IOException ex) {
                Logger.getLogger(Level.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
            }
        }
    }
    
    public Tile.Type[][] getTileLayout() {
        return tileLayout;
    }

    public int getWidth() {
        return width;
    }

    public int getHeight() {
        return height;
    }

    public int getPlayersCount() {
        return playersCount;
    }
    
    public PlayerDescription getPlayerDescription(int playerNumber) {
        if(playerNumber < 0 || playerNumber >= playersCount) {
            System.out.println("ERROR: Asking for player description with wrong player number.");
        }
        return playerDescriptions[playerNumber];
    }
    
    protected final Point[] getCharacterPosInScriptLayout(Character c) {
        ArrayList<Point> list = new ArrayList<Point>();
        
        for(int j = 0; j < scriptLayout.length; j++) {
            for(int i = 0; i < scriptLayout[j].length; i++) {
                if(scriptLayout[j][i].equals(c)) {
                    list.add(new Point(i, j));
                }
            }
        }
        
        Point[] out = new Point[list.size()];
        list.toArray(out);
        return out;
    }
}
