package com.pancik.libgdx.zumpatalentov.engine.player;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.math.Vector3;
import com.pancik.libgdx.zumpatalentov.SoundData;
import com.pancik.libgdx.zumpatalentov.engine.Engine.EngineControls;
import com.pancik.libgdx.zumpatalentov.engine.Level;
import com.pancik.libgdx.zumpatalentov.engine.building.BaseBuildingSpot;
import com.pancik.libgdx.zumpatalentov.engine.building.BuildingSpotEntity;
import com.pancik.libgdx.zumpatalentov.engine.building.MainBuilding;
import com.pancik.libgdx.zumpatalentov.engine.building.TowerBuildingSpot;
import com.pancik.libgdx.zumpatalentov.engine.component.LevelTileManager;
import com.pancik.libgdx.zumpatalentov.engine.particle.NukeExplosion;
import com.pancik.libgdx.zumpatalentov.engine.player.manager.ResourceManager;
import com.pancik.libgdx.zumpatalentov.entity.attackable.AttackableEntity;
import com.pancik.libgdx.zumpatalentov.util.MouseEvent;
import com.pancik.libgdx.zumpatalentov.util.RandomGenerator;
import java.util.ArrayList;
import java.util.List;

public abstract class PlayablePlayer extends Player {
    protected static final int SCROLL_AREA = 25;
    protected static final float SCROLL_SPEED = 15.0f;

    protected static final float RESCUE_STRIKE_RANGE = 700.0f;
    protected static final int RESCUE_STRIKE_DAMAGE = 1000;
    public static final int RESCUE_STRIKE_COOLDOWN = 1800;
    public static final ResourceManager.CostWrapper RESCUE_STRIKE_COST = new ResourceManager.CostWrapper(30, 30);
    
    protected Vector2 cameraPos = new Vector2();
    protected Vector2 mousePosition = null;

    protected MainBuilding baseBuilding = null;
    protected List<BuildingSpotEntity> buildingSpotEntities = new ArrayList<BuildingSpotEntity>();

    protected boolean rescueStrikeCooldown = false;
    protected int rescueStrikeCooldownStep = 0;
    
    public PlayablePlayer(int number, int team, EngineControls engineControls) {
        super(number, team, engineControls);
    }
        
    @Override
    public void playerTick() {
        if(baseBuilding != null) {
            if(baseBuilding.remove()) {
                baseBuilding = null;
            }
        }

        if (!Gdx.input.isButtonPressed(Input.Buttons.LEFT)
            && !Gdx.input.isButtonPressed(Input.Buttons.MIDDLE)
            && !Gdx.input.isButtonPressed(Input.Buttons.RIGHT)) {
            Vector3 mousePos = new Vector3(Gdx.input.getX(), Gdx.input.getY(), 0);
            engineControls.getUICamera().unproject(mousePos);

            if (mousePos.x < SCROLL_AREA) {
                cameraPos.add(-1.0f * SCROLL_SPEED, 0.0f);
            }

            if (mousePos.x >= Gdx.graphics.getWidth() - SCROLL_AREA) {
                cameraPos.add(1.0f * SCROLL_SPEED, 0.0f);
            }

            if (mousePos.y < SCROLL_AREA) {
                cameraPos.add(0.0f, -1.0f * SCROLL_SPEED);
            }

            if (mousePos.y > Gdx.graphics.getHeight() - SCROLL_AREA) {
                cameraPos.add(0.0f, 1.0f * SCROLL_SPEED);
            }
        }

        LevelTileManager tileManager = null;
        if (engineControls.getTileManager() instanceof LevelTileManager) {
            tileManager = (LevelTileManager) engineControls.getTileManager();
        }
        if (tileManager != null) {
            if (cameraPos.x < tileManager.getMinX()) {
                cameraPos.x = tileManager.getMinX();
            }
        }
        if (tileManager != null) {
            if (cameraPos.x > tileManager.getMaxX()) {
                cameraPos.x = tileManager.getMaxX();
            }
        }
        if (tileManager != null) {
            if (cameraPos.y < tileManager.getMinY()) {
                cameraPos.y = tileManager.getMinY();
            }
        }

        if (tileManager != null) {
            if (cameraPos.y > tileManager.getMaxY()) {
                cameraPos.y = tileManager.getMaxY();
            }
        }
        
        if(rescueStrikeCooldown) {
            rescueStrikeCooldownStep++;
            if(rescueStrikeCooldownStep >= RESCUE_STRIKE_COOLDOWN) {
                rescueStrikeCooldownStep = 0;
                rescueStrikeCooldown = false;
            }
        }
    }

    @Override
    public void playerRenderUI() {
        for(BuildingSpotEntity spot : buildingSpotEntities) {
            spot.renderUI();
        }
    }
    
    @Override
    public Vector2 getCameraPosition() {
        return cameraPos;
    }

    @Override
    public void playerHandle(MouseEvent me) {
        boolean handleForCamera = true;
        for(BuildingSpotEntity spot : buildingSpotEntities) {
            if(spot.handle(me)) {
                handleForCamera = false;
            }
        }
        
        if(baseBuilding != null && handleForCamera) {
            handleForCamera = !(baseBuilding.handle(me));
        }
        
        if(handleForCamera) {
            if(me.mouseDown()) {
                if(mousePosition == null) {
                    mousePosition = new Vector2(me.getUiUnprojectedX(), me.getUiUnprojectedY());
                }
            } else if(me.mouseDragged()) {
                if(mousePosition != null) {
                    float lastx = mousePosition.x;
                    float lasty = mousePosition.y;
                    mousePosition.set(me.getUiUnprojectedX(), me.getUiUnprojectedY());

                    cameraPos.add(-((mousePosition.x - lastx) / 1.0f), -((mousePosition.y - lasty) / 1.0f));
                } else {
                    mousePosition = new Vector2(me.getUiUnprojectedX(), me.getUiUnprojectedY());
                }
            } else if(me.mouseUp()) {
                mousePosition = null;
            }
        } else {
            mousePosition = null;
        }
    }

    @Override
    public Vector2 getSpawnPosition() {
        Level.PlayerDescription desc = engineControls.getPlayerDescription(number);
        return desc.getStartingPoint();
    }

    @Override
    public Level.WaypointPath getRandomWaypointPath() {
        Level.PlayerDescription desc = engineControls.getPlayerDescription(number);
        return desc.getWaypointPaths().get(RandomGenerator.getRandom(desc.getWaypointPaths().size()));
    }
    
    @Override
    public void loadFromDescription(Level.PlayerDescription desc) {
        cameraPos.set(desc.getBaseSpawnPoint());
        
        baseBuilding = new MainBuilding(desc.getBaseSpawnPoint(), MainBuilding.getDefaultSize(), team, this, engineControls, desc.isBaseSpawnPointFlipped());
        engineControls.addEntity(baseBuilding);
        
        for(Level.PlayerDescription.BuildingSpotInfo towerInfo : desc.getTowerBuildingSpots()) {
            TowerBuildingSpot spot = new TowerBuildingSpot(towerInfo.pos.cpy(), TowerBuildingSpot.getDefaultSize(), team, this, engineControls, towerInfo.buildingsBuildable);
            this.buildingSpotEntities.add(spot);
            
            engineControls.addEntity(spot);
        }
        
        for(Level.PlayerDescription.BuildingSpotInfo baseBuildingInfo : desc.getBaseBuildingSpots()) {
            BaseBuildingSpot spot = new BaseBuildingSpot(baseBuildingInfo.pos.cpy(), BaseBuildingSpot.getDefaultSize(), team, this, engineControls, baseBuildingInfo.buildingsBuildable);
            this.buildingSpotEntities.add(spot);
            
            engineControls.addEntity(spot);
        }
    }

    @Override
    public boolean isDefeated() {
        if(baseBuilding == null) {
            return true;
        } else {
            return baseBuilding.isDead();
        }
    }

    public void onMainBuildingHit(int remainingHealth, int maxHealth) {
    }
    
    public void onBaseBuildingHit(int remainingHealth, int maxHealth) {
    }
    
    public boolean isRescueStrikeCooldown() {
        return rescueStrikeCooldown;
    }
    
    public boolean doRescueStrike() {
        if(!rescueStrikeCooldown && baseBuilding != null) {
            if(resourceManager.hasCurrencyFor(RESCUE_STRIKE_COST)) {
                resourceManager.removeCurrency(RESCUE_STRIKE_COST);
                
                SoundData.playSound("explosion");
                
                engineControls.addParticle(new NukeExplosion(baseBuilding.getPos().sub(0, baseBuilding.getSize().x/2.0f), baseBuilding.getSize().x * 3));

                List<AttackableEntity> entities = engineControls.getEntityManager().getAttackableEntities(baseBuilding.getPos(), RESCUE_STRIKE_RANGE);
                for(AttackableEntity e : entities) {
                    if(e.getTeam() != getTeam()) {
                        e.getHit(RESCUE_STRIKE_DAMAGE, baseBuilding);
                    }
                }
                rescueStrikeCooldown = true;
                return true;
            } else {
                addErrorMessage("Insufficient funds for rescue strike!");
            }
        }       
        return false;
    }
}
