package com.pancik.libgdx.zumpatalentov.engine.player;

import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.math.Vector2;
import com.pancik.libgdx.zumpatalentov.DrawableData;
import com.pancik.libgdx.zumpatalentov.engine.Engine;
import com.pancik.libgdx.zumpatalentov.engine.Engine.EngineControls;
import com.pancik.libgdx.zumpatalentov.engine.IDObject;
import com.pancik.libgdx.zumpatalentov.engine.Level;
import com.pancik.libgdx.zumpatalentov.engine.player.manager.BonusManager;
import com.pancik.libgdx.zumpatalentov.engine.player.manager.ResourceManager;
import com.pancik.libgdx.zumpatalentov.engine.ui.Icon;
import com.pancik.libgdx.zumpatalentov.engine.ui.IconSelection;
import com.pancik.libgdx.zumpatalentov.engine.ui.Tooltip;
import com.pancik.libgdx.zumpatalentov.engine.ui.window.Window;
import com.pancik.libgdx.zumpatalentov.entity.unit.buff.Buff;
import com.pancik.libgdx.zumpatalentov.entity.unit.buff.upgrade.Upgrade;
import com.pancik.libgdx.zumpatalentov.ui.ButtonHorizontalAlignment;
import com.pancik.libgdx.zumpatalentov.ui.ButtonVerticalAlignment;
import com.pancik.libgdx.zumpatalentov.util.MouseEvent;
import com.pancik.libgdx.zumpatalentov.util.RenderUtils;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map.Entry;

public abstract class Player extends IDObject {
    private static final int ERROR_MESSAGE_LIFETIME = 240;
    private static final int ERROR_MESSAGE_MAX_COUNT = 3;
    
    private static class ErrorMessage {
        public String message;
        public int lifetime = 0;

        public ErrorMessage(String message) {
            this.message = message;
        }
        
        public boolean remove() {
            return (lifetime > ERROR_MESSAGE_LIFETIME);
        }
        
        public void tick() {
            lifetime++;
        }
        
        public void render(ButtonVerticalAlignment vertical, int deltaY) {
            if(!remove()) {
                int timeRemaining = ERROR_MESSAGE_LIFETIME - lifetime;
                if(timeRemaining < 0) {
                    timeRemaining = 0;
                }
                
                float alpha = 1.0f;
                if( timeRemaining < 60) {
                    alpha = (1.0f/60.0f) * (float)timeRemaining;
                }

                RenderUtils.blitText(DrawableData.spriteBatch,
                        message,
                        ButtonHorizontalAlignment.MIDDLE,
                        0,
                        vertical,
                        deltaY,
                        RenderUtils.TextAlligment.MIDDLE,
                        RenderUtils.TextAlligment.MIDDLE,
                        Color.RED, alpha);

                DrawableData.spriteBatch.setColor(1.0f, 1.0f, 1.0f, 1.0f);
            }
        }
    }
    
    private LinkedList<ErrorMessage> errorMessages = new LinkedList<ErrorMessage>();
    
    protected int number;
    protected int team;
    protected Engine.EngineControls engineControls;
    protected ResourceManager resourceManager;
    protected BonusManager bonusManager;
    
    protected HashMap<Class<? extends Upgrade>, Upgrade> upgrades = new HashMap<Class<? extends Upgrade>, Upgrade>();
    protected Window currentWindow = null;
    protected Tooltip tooltip = null;
    protected int tooltipLife = 0;
    protected IconSelection iconSelection = null;
    
    public Player(int number, int team, EngineControls engineControls) {
        this.number = number;
        this.team = team;
        this.engineControls = engineControls;
        this.resourceManager = new ResourceManager();
        this.bonusManager = new BonusManager(resourceManager, engineControls);
    }

    public int getNumber() {
        return number;
    }

    public int getTeam() {
        return team;
    }

    public void tick() {
        for(ErrorMessage message : errorMessages) {
            if(message.remove()) {
                errorMessages.remove(message);
                break;
            }
        }
        
        while(errorMessages.size() > ERROR_MESSAGE_MAX_COUNT) {
            errorMessages.removeFirst();
        }
        
        for(ErrorMessage message : errorMessages) {
            message.tick();
        }
        
        for(Entry<Class<? extends Upgrade>, Upgrade> entry : upgrades.entrySet()) {
            if(entry.getValue() != null) {
                entry.getValue().tick();
            }
        }
        
        for(Entry<Class<? extends Upgrade>, Upgrade> entry : upgrades.entrySet()) {
            if(entry.getValue() != null) {
                if(entry.getValue().remove() && entry.getValue().isTemporary()) {
                    entry.setValue(null);
                }
            }
        }
        
        resourceManager.tick();

        bonusManager.tick();
        
        if(iconSelection != null) {
            iconSelection.tick();
        }
        
        playerTick();
    }
    
    public void renderUI() {
        playerRenderUI();
        
        bonusManager.renderUI();
        
        if(iconSelection != null) {
            iconSelection.renderUI();
        }
        
        if(currentWindow != null) {
            currentWindow.renderUI();
        }

        //System.out.println("Tooltip is: " + tooltip + " and life is: " + tooltipLife);
        if(tooltip != null) {
            tooltip.render();
            tooltipLife++;
            if(tooltipLife > 12) {
                tooltipLife = 0;
                tooltip = null;
            }
        }
        
        //render error messages
        int deltaY = 120 - (30 * errorMessages.size());
        for(int i = 0; i < errorMessages.size(); i++) {
            errorMessages.get(i).render(ButtonVerticalAlignment.TOP, deltaY);
            deltaY += 30;
        }
        
        //render amount of coins in top left corner
        Icon.COIN.renderUI(ButtonHorizontalAlignment.LEFT, 2, ButtonVerticalAlignment.TOP, -4, 32, 32);
        
        RenderUtils.blitText(DrawableData.spriteBatch,
                Integer.toString(resourceManager.getCoinsAmount()) + " + " + Integer.toString(resourceManager.getIncomePerMinute()),
                ButtonHorizontalAlignment.LEFT,
                32 + 4,
                ButtonVerticalAlignment.TOP,
                2,
                RenderUtils.TextAlligment.LEFT,
                RenderUtils.TextAlligment.BOTTOM);
        
        Icon.TOFU.renderUI(ButtonHorizontalAlignment.LEFT, 2, ButtonVerticalAlignment.TOP, 22, 32, 32);
        
        RenderUtils.blitText(DrawableData.spriteBatch,
                Integer.toString(resourceManager.getBonusCurrencyAmount()),
                ButtonHorizontalAlignment.LEFT,
                32 + 2,
                ButtonVerticalAlignment.TOP,
                28,
                RenderUtils.TextAlligment.LEFT,
                RenderUtils.TextAlligment.BOTTOM);
    }
    
    public ResourceManager getResourceManager() {
        return resourceManager;
    }
    
    public BonusManager getBonusManager() {
        return bonusManager;
    }
 
    public void addErrorMessage(String message) {
        errorMessages.add(new ErrorMessage(message));
    }
    
    public void setCurrentWindowAndShow(Window w) {
        if(w != null) {
            w.hide();
        }
        this.currentWindow = w;
        this.currentWindow.show();
    }
    
    public void handle(MouseEvent me) {
        boolean handle = true;
        if(currentWindow != null) {
            if ((!currentWindow.handle(me) && me.mouseDown()) || currentWindow.isHidden()) {
                currentWindow.hide();
            } else {
                handle = false;
            }
        }

        if(currentWindow != null) {
            if((me.mouseDragged() || me.mouseUp()) && !currentWindow.isHidden()) {
                handle = false;
            }
        }
        
        if(handle) {
            handle = !(bonusManager.handle(me));

            if(iconSelection != null && handle) {
                iconSelection.handle(me);
            }

            if(handle) {
                playerHandle(me);
            }
        }
    }
            
    public void setTooltip(Tooltip t) {
        this.tooltip = t;
    }
    
    public void setIconSelectionAndShow(IconSelection selection) {
        if(this.iconSelection != null) {
            this.iconSelection.hide();
        }
        this.iconSelection = selection;
        this.iconSelection.show();
    }
    
    public void addUpgrade(Upgrade upgrade) {
        upgrades.put(upgrade.getClass(), upgrade);
    }
    
    public float getUpgradeCoefficient(Buff.PercentageStat stat) {
        float out = 1.0f;
        
        for(Entry<Class<? extends Upgrade>, Upgrade> entry : upgrades.entrySet()) {
            if(entry.getValue() != null) {
                if(!entry.getValue().remove()) {
                    out += entry.getValue().getCoefficient(stat);
                }
            }
        }
        
        return out;
    }
    
    public abstract boolean isLocal();
    public abstract boolean isDefeated();
    public abstract void playerTick();
    public abstract void playerRenderUI();
    public abstract Vector2 getCameraPosition();
    public abstract void playerHandle(MouseEvent me);
    public abstract void loadFromDescription(Level.PlayerDescription desc);
    public abstract Vector2 getSpawnPosition();
    public abstract Level.WaypointPath getRandomWaypointPath();
}
